{{
LineRAM based sprite display for 240H composite NTSC output
(c) Eric Ball Creative Commons Attribution-ShareAlike 3.0 License
2007-05-06 genesys of idea
2007-05-26 pre-alpha internal testing release (very buggy)
2007-07-06 alpha release to forums.parallax.com of All-In-One testcode

System clock must be > 62MHz

The idea for this sprite routine is to copy the sprite data to a cog RAM based
LineRAM buffer. Each long in the buffer is one cycle of colorburst (four
14,318,182Hz luma samples). The output routine outputs 3 luma samples per 
pixel and is capable of producing most/all of the NTSC color gamut (depending
on the number of bits in the output DAC). This code assumes demoboard / Hydra
composite output and is capable of a palette of 81 colors (plus transparent)
out of 172 possible. See NTSC240.HTML for hovertext CLUT values.

Multiple cogs (2-5) run the same code and are syncronized to output lines
sequentially (i.e. while cog 1 is outputting line 1 from LineRAM, cog 2 is
creating line 2.) to create a 240x240 image in a standard NTSC non-interlaced
frame (303.3 pixels x 262 lines). The size of the sprite table and the number
of active sprites is dependent on the number of cogs and the system clock.

Sprite Table format addr[16]:xpos[8]:ypos[8] (lsb) (all 0 for unused)
Sprites are 8x8 pixels (1 byte/pixel) stored in raster order
color #0 is transparent, all others are color index into cog RAM
xpos MUST be limitted to 0-232 - no wrap around or scrolling!
vertical scrolling is possible (i.e. ypos=255 will scroll in from top,
ypos=233 will scroll off bottom)
current row number (240=vsync/vblank) stored in byte after Sprite Table
Note: sprite table and sprite graphics must be long aligned
}}
CON
  MAXCOGS = 5
  ' Set up the processor clock in the standard way for 80MHz
  _XINFREQ = 10_000_000 + 0000
  _CLKMODE = xtal1 + pll8x
  
VAR
  byte vidcog[MAXCOGS]

PUB start | cog, i

'' Start LineRAM cogs
'' Input: parmptr = pointer to parameter table
'' Ouput: false (none started) if CLKFREQ too low or not enough cogs 

  stop                                                  ' stop any existing cogs
  if CLKFREQ < 62045455                                 ' colorburst * 4 / 3 * 13
    RETURN FALSE

  sprite_tab.BYTE[0] := 0       ' ypos, sprite[0]
  sprite_tab.BYTE[1] := 0       ' xpos, sprite[0]
  sprite_tab.WORD[1] := @sprite
{
  sprite_tab.BYTE[4] := 0                 ' ypos, sprite[1]
  sprite_tab.BYTE[5] := Constant(240-8)   ' xpos, sprite[1]
  sprite_tab.WORD[3] := @sprite

  sprite_tab.BYTE[8] := Constant(240-8)   ' ypos, sprite[2]
  sprite_tab.BYTE[9] := Constant(240-8)   ' xpos, sprite[2]
  sprite_tab.WORD[5] := @sprite

  sprite_tab.BYTE[12] := Constant(240-8)  ' ypos, sprite[3]
  sprite_tab.BYTE[13] := 0                ' xpos, sprite[3]
  sprite_tab.WORD[7] := @sprite
}
  sprite_tab.BYTE[16] := Constant((240-8)/2) ' ypos, sprite[4]
  sprite_tab.BYTE[17] := Constant((240-8)/2) ' xpos, sprite[4]
  sprite_tab.WORD[9] := @sprite

  N2_PLLAFRQ := 0   ' FRQA for 7,159,091 Hz, set to 0 to auto-calculate
  N2_OUTPINS := %001_011110000  ' demoboard = %001_011110000, Hydra = %011_000001111
  N2_NUMCOGS := 2   ' number of cogs (2-5)
  N2_TBLSIZE := 64   ' number of entries in sprite table
  N2_MAXSPR  := 8   ' maximum number of active sprites on a single line
  N2_TBLPTR  := @sprite_tab   ' pointer to sprite table
  count2 := 10000 + 1000 * N2_NUMCOGS               ' synchronization CNT
  count2 += CNT

  repeat i from 1 to N2_NUMCOGS
    if (cog := COGNEW( @sync, N2_TBLPTR )) < 0 ' no cogs available
      stop                                              ' stop anything started
      RETURN FALSE
    else
      cogid0num[cog] := i                               ' save cog index (works 'cause COGNEW is not instant)
      vidcog[i-1] := cog + 1                              ' save cogid
  RETURN TRUE

PUB stop | i

'' Stop LineRAM cogs

  repeat i from 0 to CONSTANT(MAXCOGS-1)
    if vidcog[i]
       COGSTOP( vidcog[i]~ - 1 )                        ' stop cog & set vidcog to 0

DAT
                        ORG $0
sync                    JMP     #initall                ' 20IRE -20IRE -40IRE 0IRE (lsb)
'                       LONG    Y+V_Y+U_Y-V_Y-U         ' CLUT format bit[7-4] = bit[3-0] = 40IRE 80IRE 40IRE 20IRE (lsb)
black                   LONG    $22222222               ' first entry in CLUT, space for 80 more
                        LONG    $33333333               ' greyscale
                        LONG    $44444444
                        LONG    $55555555
                        LONG    $66666666
                        LONG    $77777777               ' 100 IRE white!

nextrow                 WAITVID sync, #0                ' front porch (blank)
nextrow2                MOV     VSCL, vscl910           ' stretch to NUMCOGS-1 lines per WAITVID
                        WAITVID sync, #%%11111          ' output -40 IRE (outputs low)
                        MOV     DIRA, #0                ' disable output
numcogs1                ADD     rownum, #N2_NUMCOGS
                        CMPSUB  rownum, #262
initrow                 CMP     rownum, #240    WC,WZ
        IF_C            JMP     #doactive
                        CMP     rownum, #250    WC
        IF_NC_OR_Z      JMP     #doblank
                        CMP     rownum, #244    WZ
        IF_NZ           CMP     rownum, #245    WZ
        IF_NZ           CMP     rownum, #246    WZ

dovsync                 MOV     count1, #2
:half   IF_NZ           MOV     VSCL, vscl33            ' 33 PLLA @ -40 IRE (equalizing pulse)
        IF_Z            MOV     VSCL, vscl388           ' 388 PLLA @ -40 IRE (serration pulse)
                        WAITVID sync, #%%11111          ' start at end of current line
                        MOV     DIRA, viddira           ' enable output
        IF_NZ           MOV     VSCL, vscl422           ' 422 PLLA @ 0 IRE (equalizing pulse)
        IF_Z            MOV     VSCL, vscl67            ' 67 PLLA @ 0 IRE (serration pulse)
                        WAITVID sync, #0
                        DJNZ    count1, #:half
                        JMP     #nextrow2

doactive                MOVD    :clear, #lineram        ' reset destination
                        MOV     count1, #240            ' number of pixels in LineRAM
:clear                  MOV     lineram, black          ' background color
                        ADD     :clear, d1
                        DJNZ    count1, #:clear
tblsize1                MOV     count1, #N2_TBLSIZE     ' number of entries in sprite table
maxspr1                 MOV     count2, #N2_MAXSPR      ' maximum number of active sprites
                        MOV     sprptr, PAR
:loop                   RDLONG  sprdata, sprptr WZ      ' 7  get sprite table entry
        IF_Z            JMP     #:nxtspr                ' 11 zero entry, go to next sprite
                        MOV     sprtemp, rownum         ' 15 format addr[16]:xpos[8]:ypos[8]
                        SUB     sprtemp, sprdata        ' 19 relative to current row
                        TEST    sprtemp, #$0F8  WZ      ' 23 check for outside 0-7
        IF_NZ           JMP     #:nxtspr                ' 27 outside range, go to next sprite
                        MOV     spraddr, sprdata        ' 31 calculate sprite address
                        SHR     spraddr, #16            ' 35 base address for top left
                        AND     sprtemp, #7             ' 39 mask off offset 
                        SHL     sprtemp, #3             ' 43 8 bytes per row
                        ADD     spraddr, sprtemp        ' 47
                        RDLONG  sprtemp, spraddr        ' 48->7
                        ADD     spraddr, #4             ' 11
                        SHR     sprdata, #8             ' 15 shift xpos to lsb
                        RDLONG  spraddr, spraddr        ' 16->7
                        AND     sprdata, #$0FF          ' 11 mask off xpos
                        ADD     sprdata, #lineram       ' 15 add LineRAM base address

                        MOV     sprbyte, sprtemp        ' handle each byte (LSB first!)
                        AND     sprbyte, #$0FF  WZ      ' $00 is transparent
                        MOVS    :byte0, sprbyte         ' set source
                        MOVD    :byte0, sprdata         ' set destination
                        ADD     sprdata, #1
:byte0  IF_NZ           MOV     sprdata, sprbyte
                        SHR     sprtemp, #8
                        MOV     sprbyte, sprtemp
                        AND     sprbyte, #$0FF  WZ
                        MOVS    :byte1, sprbyte
                        MOVD    :byte1, sprdata
                        ADD     sprdata, #1
:byte1  IF_NZ           MOV     sprdata, sprbyte
                        SHR     sprtemp, #8
                        MOV     sprbyte, sprtemp
                        AND     sprbyte, #$0FF  WZ
                        MOVS    :byte2, sprbyte
                        MOVD    :byte2, sprdata
                        ADD     sprdata, #1
:byte2  IF_NZ           MOV     sprdata, sprbyte
                        SHR     sprtemp, #8     WZ
                        MOVS    :byte3, sprtemp
                        MOVD    :byte3, sprdata
                        ADD     sprdata, #1
:byte3  IF_NZ           MOV     sprdata, sprbyte
                        MOV     sprbyte, spraddr
                        AND     sprbyte, #$0FF  WZ
                        MOVS    :byte4, sprbyte
                        MOVD    :byte4, sprdata
                        ADD     sprdata, #1
:byte4  IF_NZ           MOV     sprdata, sprbyte
                        SHR     spraddr, #8
                        MOV     sprbyte, spraddr
                        AND     sprbyte, #$0FF  WZ
                        MOVS    :byte5, sprbyte
                        MOVD    :byte5, sprdata
                        ADD     sprdata, #1
:byte5  IF_NZ           MOV     sprdata, sprbyte
                        SHR     spraddr, #8
                        MOV     sprbyte, spraddr
                        AND     sprbyte, #$0FF  WZ
                        MOVS    :byte6, sprbyte
                        MOVD    :byte6, sprdata
                        ADD     sprdata, #1
:byte6  IF_NZ           MOV     sprdata, sprbyte
                        SHR     spraddr, #8     WZ
                        MOVS    :byte7, spraddr
                        MOVD    :byte7, sprdata
                        DJNZ    count2, #:byte7
                        MOV     :byte7b, :byte7
                        JMP     #:byte7b
:byte7  IF_NZ           MOV     sprdata, sprbyte
:nxtspr                 ADD     sprptr, #4
                        DJNZ    count1, #:loop  WZ      ' 48+16+223 = 287 (288) max cycles / sprite
:byte7b IF_NZ           MOV     sprdata, sprbyte

doblank                 MOV     VSCL, vscl67            ' 67 PLLA @ -40 IRE (sync pulse)
                        WAITVID sync, #%%11111          ' start at end of current line
                        MOV     DIRA, viddira           ' enable outputs
                        CMP     rownum, #240    WC,WZ   ' active or blank
        IF_C_OR_Z       WRBYTE  rownum, outptr          ' write rownum to main memory (240 is vsync/vblank)
                        TEST    rownum, #1      WZ      ' even or odd row
        IF_Z            MOVS    :burst, #%%0203         ' -U burst
        IF_NZ           MOVS    :burst, #%%0302         ' +U burst
        IF_Z            MOV     :pixel0, evenp0         ' set colorburst phase & init pointers
        IF_NZ           MOV     :pixel0, oddp0
        IF_Z            MOV     :pixel1, evenp1
        IF_NZ           MOV     :pixel1, oddp1
        IF_Z            MOV     :pixel2, evenp2
        IF_NZ           MOV     :pixel2, oddp2
        IF_Z            MOV     :pixel3, evenp3
        IF_NZ           MOV     :pixel3, oddp3
                        MOV     VSCL, vscl9             ' reset back to 1 PLLA per pixel
                        WAITVID sync, #0                ' 9 PLLA @ 0 IRE (blank)
                        MOV     count1, #9
                        MOVS    VSCL, #4                ' 4 pixels (one cycle) per frame
:burst                  WAITVID sync, #%%0302           ' 36 PLLA: 9 cycles colorburst
                        DJNZ    count1, #:burst
                        MOVS    VSCL, #20
                        WAITVID sync, #0                ' 20 PLLA @ 0 IRE (blank)
        IF_NC           MOV     VSCL, vscl778
        IF_NC           JMP     #nextrow                ' 778 PLLA @ 0 IRE (blank)
                        MOVS    VSCL, #18
                        WAITVID black, #0               ' 18 PLLA @ 7.5 IRE (black)
                        MOV     count1, #240/4
                        MOVS    VSCL, #3                ' 3 pixels per frame (240H)
:active                 ADD     :pixel0, d4             ' 720 PLLA active video (240 pixels)
:pixel0                 WAITVID lineram, #%%230         ' Y-U Y+V Y+U
                        ADD     :pixel1, d4             ' note: doesn't take effect until next loop
:pixel1                 WAITVID lineram+1, #%%301       ' Y-V Y-U Y+V
                        ADD     :pixel2, d4
:pixel2                 WAITVID lineram+2, #%%012       ' Y+U Y-V Y-U
                        ADD     :pixel3, d4
:pixel3                 WAITVID lineram+3, #%%123       ' Y+V Y+U Y-V
                        DJNZ    count1, #:active
                        WAITVID black, #0               ' 3 PLLA @ 7.5 IRE (black)
                        MOVS    VSCL, #16
                        WAITVID black, #0               ' 16 PLLA @ 7.5 IRE (black)
                        MOVS    VSCL, #21
                        JMP     #nextrow                ' 21 PLLA @ 0 IRE (blank)

evenp0                  WAITVID lineram, #%%012         ' +U -V -U
evenp1                  WAITVID lineram+1, #%%123       ' +V +U -V
evenp2                  WAITVID lineram+2, #%%230       ' -U +V +U
evenp3                  WAITVID lineram+3, #%%301       ' -V -U +V

oddp0                   WAITVID lineram, #%%230         ' -U +V +U
oddp1                   WAITVID lineram+1, #%%301       ' -V -U +V
oddp2                   WAITVID lineram+2, #%%012       ' +U -V -U
oddp3                   WAITVID lineram+3, #%%123       ' +V +U -V

d1                      LONG    1<<9                    ' opcode Dest field = 1
d4                      LONG    4<<9                    ' opcode Dest field = 4
vscl9                   LONG    1<<12+9                 ' 1 PLLA per pixel, 9 PLLA per frame
vscl33                  LONG    33<<12+33               ' 33 PLLA per pixel/frame
vscl67                  LONG    67<<12+67               ' 67 PLLA per pixel/frame
vscl388                 LONG    194<<12+388             ' 388 PLLA per frame
vscl422                 LONG    211<<12+422             ' 422 PLLA per frame
vscl778                 LONG    1<<12+778               ' 778 PLLA per frame
vscl910                 LONG    182<<12-910             ' 910*(NUMCOGS-1) PLLA per frame (line)
viddira                 LONG    %1111<<12               ' output on pins 12-15

count1                  LONG    $63697245               ' multipurpose counter
count2                  LONG    $6C6C6142               ' maximum number of active sprites
outptr                  LONG    $63697245               ' pointer to end of sprite table (in main memory)
rownum                  LONG    $6C6C6142               ' current row (0 = first active row)
sprptr                  LONG    $63697245               ' pointer to sprite table entry (in main memory)
sprdata                 LONG    $6C6C6142               ' sprite table entry / pointer to lineRAM (xPos)
sprtemp                 LONG    $63697245               ' Ypos / left 4 pixels (LSByte = leftmost)
spraddr                 LONG    $6C6C6142               ' pointer to sprite graphics (main memory)
                                                        ' / right 4 pixels (MSByte = rightmost)
sprbyte                 LONG    $63697245               ' current pixel

                        FIT     $100
lineram                 LONG    $33110022               ' 20IRE -20IRE -40IRE 0IRE (lsb)
initall                 MOV     sync, lineram
                        COGID   count1
                        ADD     :cognum, count1         ' load cognum
                        MOV     rownum, #242            ' init rownum = 239 + cognum
:cognum                 MOV     count1, cogid0num       ' count1 = cogid0num[COGID]
                        ADD     rownum, count1
:i455                   ADD     ivscl, l910             ' calculate initial delay
                        DJNZ    count1, #:i455
                        MOV     outptr, N2_TBLSIZE      ' init outptr
                        SHL     outptr, #4
                        ADD     outptr, PAR             ' init outptr
                        MOVS    tblsize1, N2_TBLSIZE    ' initialize code constants
                        MOVS    maxspr1, N2_MAXSPR
                        MOVS    numcogs1, N2_NUMCOGS
                        MOV     count1, N2_NUMCOGS      ' calculate row delay
:i910                   ADD     vscl910, l910
                        DJNZ    count1, #:i910
                        MOV     viddira, N2_OUTPINS     ' calculate DIRA
                        AND     viddira, #$FF
                        MOV     count1, N2_OUTPINS
                        SHR     count1, #9
                        SHL     count1, #3
                        SHL     viddira, count1
                        MOVI    N2_OUTPINS, #%001100000 ' 4 color VGA
                        TJNZ    N2_PLLAFRQ, #:cont      ' FRQA given or calculate?
                        RDLONG  count1, #0              ' CLKFREQ
:shmax                  SHL     l9090909, #1    WC      ' maximize the two values
                        RCL     l7159090, #1
                        SHL     count1, #1      WC
        IF_NC           JMP     #:shmax
                        RCR     count1, #1              ' undo overshoot
:div                    CMPSUB  l7159090, count1 WC     ' do division
                        RCL     N2_PLLAFRQ, #1
                        SHR     count1, #1      WZ
        IF_NZ           JMP     #:div
:cont                   WAITCNT count2, #0              ' sync to other cogs
                        MOV     CTRA, ictra             ' initialize registers
                        MOV     FRQA, N2_PLLAFRQ        ' start the counters!
                        MOV     VCFG, N2_OUTPINS
                        MOV     VSCL, ivscl
                        WAITVID sync, #0                ' initial delay
                        JMP     #initrow

l910                    LONG    910
l7159090                LONG    $6D3D32                 ' 7159090 Hz
l9090909                LONG    $E8BA2E8B               ' 0.909090909 Hz
ictra                   LONG    %00001_100<<23          ' PLL internal, VCO/8 (PLLA = 14,318,182 Hz)
ivscl                   LONG    91<<12-455              ' 910*cognum-455 PLLA initial delay

' input parameters
N2_PLLAFRQ              LONG    0                       ' FRQA for 7,159,091 Hz, set to 0 to auto-calculate
N2_OUTPINS              LONG    %001_011110000          ' demoboard = %001_011110000, Hydra = %011_000001111
N2_NUMCOGS              LONG    2                       ' number of cogs (2-5)
N2_TBLSIZE              LONG    64                      ' number of entries in sprite table
N2_MAXSPR               LONG    8                       ' maximum number of active sprites on a single line
N2_TBLPTR               LONG    0                       ' pointer to sprite table
cogid0num               LONG    0                       ' cognum table
cogid1num               LONG    0
cogid2num               LONG    0
cogid3num               LONG    0
cogid4num               LONG    0
cogid5num               LONG    0
cogid6num               LONG    0
cogid7num               LONG    0

sprite          LONG    $05000000
                LONG    $00000005
                LONG    $02040500
                LONG    $00040301
                LONG    $06060400
                LONG    $00050101
                LONG    $05060405
                LONG    $04050101
                LONG    $01040405
                LONG    $03060301
                LONG    $05050400
                LONG    $00060605
                LONG    $06060400
                LONG    $00030606
                LONG    $04000000
                LONG    $03000000
                
sprite_tab    long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              long      0
              byte      0


{{


possible enhancement ideas:
- roll up sprite->lineram routine for larger palette (fewer sprites)
- change sprite->lineram to 6bpp for better HUB-RAM usage
- separate Y/C DACs for S-Video output (CLUT change only)
- move CLUT to main RAM (slow!) for more code space 
  - add background tiles (color translate at end of line)
  - sprite flip (rotate?) encoded in address [0..1] lsb
  - larger sprites / tiles (use internal font?)
  - built-in audio/music output on per line/frame
    - per cog voice based on CMPSUB rollover
  - background scrolling, horizontal sprite scrolling
  - closed caption generation
- PAL output
  - PLLA = 17,734,475 Hz (77MHz minimum system clock)
  - 1135 PLLA per line, 312 lines per frame
  - CLUT will be wrt colorburst (45 degrees), not U/V (different colors!)
  - odd/even line needs to account for colorburst V swing
  - different hsync/vsync timing
    28 blank, 74 sync, 11 blank, 36 burst, 20 blank
    equalizing: 36/531/36/531, serration: 498/69/498/69
  - more blank lines (256 out of 288 possible), or extend Y to 9 bits
  - 975 PLLA active per line, either very narrow screen or 4 samples/pixel
    and picture extends into overscan

Notes:
- 5 cog limit is due to 12 bits for VSCL frame counter (4 for PAL)
- the code compensates for CLUT samples MSB = phase 0 while LSB = color 0
- broadcast is not supported
- CLUT values should follow these rules:
  - samples between -40 IRE (all off) and 120 IRE
  - Y constant for all 4 values & between 0 and 100 IRE
  - RGB output between 0.0 & 1.0 using the following formulas:
    R = Y / 100 + V / 162.245
    G = Y / 100 - U / 468.673 - V / 318.521
    B = Y / 100 + U / 91.02
  - CLUT values which do not follow these rules may exceed NTSC specs or 
    produce unstable pixels

Thanks to Doug Dingus (aka potatohead) for starting me on this think exercise
and for debugging support.  And a very big thank you to Robert Bryon Vandiver 
(aka asterick) for Gear and mirror for his breakpoint modification.  Without them I would have been debugging blind.
}}

{ NTSC240.HTML
<html><head>
</head><body>
<table width="100%" border="1"><tr>
<td bgcolor="#000000" title= "CLUT:22222222">&nbsp;</td>
<td bgcolor="#333333" title= "CLUT:33333333">&nbsp;</td>
<td bgcolor="#666666" title= "CLUT:44444444">&nbsp;</td>
<td bgcolor="#999999" title= "CLUT:55555555">&nbsp;</td>
<td bgcolor="#CCCCCC" title= "CLUT:66666666">&nbsp;</td>
<td bgcolor="#FFFFFF" title= "CLUT:77777777">&nbsp;</td>
</tr><tr>
<td bgcolor="#66702D" title= "CLUT:44334455">&nbsp;</td>
<td bgcolor="#99A360" title= "CLUT:55445566">&nbsp;</td>
<td bgcolor="#99AE28" title= "CLUT:55335577">&nbsp;</td>
<td bgcolor="#CCD693" title= "CLUT:66556677">&nbsp;</td>
<td bgcolor="#CCE15B" title= "CLUT:664466EE">&nbsp;</td>
</tr><tr>
<td bgcolor="#8F872B" title= "CLUT:55334466">&nbsp;</td>
<td bgcolor="#C2BA5E" title= "CLUT:66445577">&nbsp;</td>
<td bgcolor="#B89E28" title= "CLUT:66334477">&nbsp;</td>
<td bgcolor="#C2C526" title= "CLUT:663355EE">&nbsp;</td>
<td bgcolor="#F5ED91" title= "CLUT:775566EE">&nbsp;</td>
<td bgcolor="#EBD15B" title= "CLUT:774455EE">&nbsp;</td>
<td bgcolor="#E1B526" title= "CLUT:773344EE">&nbsp;</td>
</tr><tr>
<td bgcolor="#5C4930" title= "CLUT:44333344">&nbsp;</td>
<td bgcolor="#8F7C63" title= "CLUT:55444455">&nbsp;</td>
<td bgcolor="#85602D" title= "CLUT:55333355">&nbsp;</td>
<td bgcolor="#C2AF96" title= "CLUT:66555566">&nbsp;</td>
<td bgcolor="#B89360" title= "CLUT:66444466">&nbsp;</td>
<td bgcolor="#AE772B" title= "CLUT:66333366">&nbsp;</td>
<td bgcolor="#F5E2C9" title= "CLUT:77666677">&nbsp;</td>
<td bgcolor="#EBC693" title= "CLUT:77555577">&nbsp;</td>
<td bgcolor="#E1AA5E" title= "CLUT:77444477">&nbsp;</td>
<td bgcolor="#D78E28" title= "CLUT:77333377">&nbsp;</td>
</tr><tr>
<td bgcolor="#7B3930" title= "CLUT:55332244">&nbsp;</td>
<td bgcolor="#9B2930" title= "CLUT:66331144">&nbsp;</td>
<td bgcolor="#AE6C63" title= "CLUT:66443355">&nbsp;</td>
<td bgcolor="#A4502D" title= "CLUT:66332255">&nbsp;</td>
<td bgcolor="#CE5C63" title= "CLUT:77442255">&nbsp;</td>
<td bgcolor="#E19F96" title= "CLUT:77554466">&nbsp;</td>
<td bgcolor="#C4402D" title= "CLUT:77331155">&nbsp;</td>
<td bgcolor="#D78360" title= "CLUT:77443366">&nbsp;</td>
<td bgcolor="#CE672B" title= "CLUT:77332266">&nbsp;</td>
<td bgcolor="#E3302D" title= "CLUT:EE330055">&nbsp;</td>
<td bgcolor="#F77360" title= "CLUT:EE442266">&nbsp;</td>
<td bgcolor="#ED572B" title= "CLUT:EE331166">&nbsp;</td>
<td bgcolor="#F77E28" title= "CLUT:EE332277">&nbsp;</td>
</tr><tr>
<td bgcolor="#522233" title= "CLUT:44332233">&nbsp;</td>
<td bgcolor="#711233" title= "CLUT:55331133">&nbsp;</td>
<td bgcolor="#855566" title= "CLUT:55443344">&nbsp;</td>
<td bgcolor="#910233" title= "CLUT:66330033">&nbsp;</td>
<td bgcolor="#A44566" title= "CLUT:66442244">&nbsp;</td>
<td bgcolor="#B88899" title= "CLUT:66554455">&nbsp;</td>
<td bgcolor="#C43566" title= "CLUT:77441144">&nbsp;</td>
<td bgcolor="#D77899" title= "CLUT:77553355">&nbsp;</td>
<td bgcolor="#EBBBCC" title= "CLUT:77665566">&nbsp;</td>
<td bgcolor="#BA1930" title= "CLUT:77330044">&nbsp;</td>
<td bgcolor="#BA0F68" title= "CLUT:77440033">&nbsp;</td>
<td bgcolor="#E32566" title= "CLUT:EE440044">&nbsp;</td>
<td bgcolor="#F76899" title= "CLUT:EE552255">&nbsp;</td>
<td bgcolor="#ED4C63" title= "CLUT:EE441155">&nbsp;</td>
<td bgcolor="#ED429B" title= "CLUT:EE551144">&nbsp;</td>
</tr><tr>
<td bgcolor="#7B2F68" title= "CLUT:55442233">&nbsp;</td>
<td bgcolor="#71086B" title= "CLUT:55441122">&nbsp;</td>
<td bgcolor="#9B1F68" title= "CLUT:66441133">&nbsp;</td>
<td bgcolor="#AE629B" title= "CLUT:66553344">&nbsp;</td>
<td bgcolor="#A43B9E" title= "CLUT:66552233">&nbsp;</td>
<td bgcolor="#9B14A0" title= "CLUT:66551122">&nbsp;</td>
<td bgcolor="#CE529B" title= "CLUT:77552244">&nbsp;</td>
<td bgcolor="#E195CE" title= "CLUT:77664455">&nbsp;</td>
<td bgcolor="#C42B9E" title= "CLUT:77551133">&nbsp;</td>
<td bgcolor="#D76ED1" title= "CLUT:77663344">&nbsp;</td>
<td bgcolor="#BA04A0" title= "CLUT:77550022">&nbsp;</td>
<td bgcolor="#CE47D3" title= "CLUT:77662233">&nbsp;</td>
<td bgcolor="#C420D6" title= "CLUT:77661122">&nbsp;</td>
<td bgcolor="#E31B9E" title= "CLUT:EE550033">&nbsp;</td>
<td bgcolor="#F75ED1" title= "CLUT:EE662244">&nbsp;</td>
<td bgcolor="#ED37D3" title= "CLUT:EE661133">&nbsp;</td>
<td bgcolor="#E310D6" title= "CLUT:EE660022">&nbsp;</td>
</tr><tr>
<td bgcolor="#290C35" title= "CLUT:33332222">&nbsp;</td>
<td bgcolor="#5C3F68" title= "CLUT:44443333">&nbsp;</td>
<td bgcolor="#52186B" title= "CLUT:44442222">&nbsp;</td>
<td bgcolor="#8F729B" title= "CLUT:55554444">&nbsp;</td>
<td bgcolor="#854B9E" title= "CLUT:55553333">&nbsp;</td>
<td bgcolor="#7B24A0" title= "CLUT:55552222">&nbsp;</td>
<td bgcolor="#C2A5CE" title= "CLUT:66665555">&nbsp;</td>
<td bgcolor="#B87ED1" title= "CLUT:66664444">&nbsp;</td>
<td bgcolor="#AE57D3" title= "CLUT:66663333">&nbsp;</td>
<td bgcolor="#A430D6" title= "CLUT:66662222">&nbsp;</td>
<td bgcolor="#9B09D8" title= "CLUT:66661111">&nbsp;</td>
</tr><tr>
<td bgcolor="#29016D" title= "CLUT:33442211">&nbsp;</td>
<td bgcolor="#5C34A0" title= "CLUT:44553322">&nbsp;</td>
<td bgcolor="#520DA3" title= "CLUT:44552211">&nbsp;</td>
<td bgcolor="#5C29D8" title= "CLUT:44663311">&nbsp;</td>
<td bgcolor="#8F67D3" title= "CLUT:55664433">&nbsp;</td>
<td bgcolor="#5202DB" title= "CLUT:44662200">&nbsp;</td>
<td bgcolor="#8540D6" title= "CLUT:55663322">&nbsp;</td>
<td bgcolor="#7B19D8" title= "CLUT:55662211">&nbsp;</td>
</tr><tr>
<td bgcolor="#33286B" title= "CLUT:33443322">&nbsp;</td>
<td bgcolor="#331DA3" title= "CLUT:33553311">&nbsp;</td>
<td bgcolor="#665B9E" title= "CLUT:44554433">&nbsp;</td>
<td bgcolor="#3312DB" title= "CLUT:33663300">&nbsp;</td>
<td bgcolor="#6650D6" title= "CLUT:44664422">&nbsp;</td>
<td bgcolor="#998ED1" title= "CLUT:55665544">&nbsp;</td>
</tr><tr>
<td bgcolor="#09116D" title= "CLUT:22443311">&nbsp;</td>
<td bgcolor="#0906A5" title= "CLUT:22553300">&nbsp;</td>
<td bgcolor="#3C44A0" title= "CLUT:33554422">&nbsp;</td>
<td bgcolor="#132DA3" title= "CLUT:22554411">&nbsp;</td>
<td bgcolor="#3C39D8" title= "CLUT:33664411">&nbsp;</td>
<td bgcolor="#6F77D3" title= "CLUT:44665533">&nbsp;</td>
<td bgcolor="#1322DB" title= "CLUT:22664400">&nbsp;</td>
<td bgcolor="#4660D6" title= "CLUT:33665522">&nbsp;</td>
<td bgcolor="#1D49D8" title= "CLUT:22665511">&nbsp;</td>
</tr><tr>
<td bgcolor="#091C35" title= "CLUT:22333322">&nbsp;</td>
<td bgcolor="#3C4F68" title= "CLUT:33444433">&nbsp;</td>
<td bgcolor="#13386B" title= "CLUT:22444422">&nbsp;</td>
<td bgcolor="#6F829B" title= "CLUT:44555544">&nbsp;</td>
<td bgcolor="#466B9E" title= "CLUT:33555533">&nbsp;</td>
<td bgcolor="#1D54A0" title= "CLUT:22555522">&nbsp;</td>
<td bgcolor="#A2B5CE" title= "CLUT:55666655">&nbsp;</td>
<td bgcolor="#799ED1" title= "CLUT:44666644">&nbsp;</td>
<td bgcolor="#5087D3" title= "CLUT:33666633">&nbsp;</td>
<td bgcolor="#2770D6" title= "CLUT:22666622">&nbsp;</td>
</tr><tr>
<td bgcolor="#1D5F68" title= "CLUT:22445533">&nbsp;</td>
<td bgcolor="#50929B" title= "CLUT:33556644">&nbsp;</td>
<td bgcolor="#277B9E" title= "CLUT:22556633">&nbsp;</td>
<td bgcolor="#30A29B" title= "CLUT:22557744">&nbsp;</td>
<td bgcolor="#83C5CE" title= "CLUT:44667755">&nbsp;</td>
<td bgcolor="#078B9E" title= "CLUT:11557733">&nbsp;</td>
<td bgcolor="#5AAED1" title= "CLUT:33667744">&nbsp;</td>
<td bgcolor="#3097D3" title= "CLUT:22667733">&nbsp;</td>
<td bgcolor="#0780D6" title= "CLUT:11667722">&nbsp;</td>
<td bgcolor="#63D5CE" title= "CLUT:3366EE55">&nbsp;</td>
<td bgcolor="#3ABED1" title= "CLUT:2266EE44">&nbsp;</td>
<td bgcolor="#11A7D3" title= "CLUT:1166EE33">&nbsp;</td>
</tr><tr>
<td bgcolor="#134333" title= "CLUT:22334433">&nbsp;</td>
<td bgcolor="#467666" title= "CLUT:33445544">&nbsp;</td>
<td bgcolor="#278666" title= "CLUT:22446644">&nbsp;</td>
<td bgcolor="#79A999" title= "CLUT:44556655">&nbsp;</td>
<td bgcolor="#079666" title= "CLUT:11447744">&nbsp;</td>
<td bgcolor="#5AB999" title= "CLUT:33557755">&nbsp;</td>
<td bgcolor="#ACDCCC" title= "CLUT:55667766">&nbsp;</td>
<td bgcolor="#3AC999" title= "CLUT:2255EE55">&nbsp;</td>
<td bgcolor="#8DECCC" title= "CLUT:4466EE66">&nbsp;</td>
<td bgcolor="#11B29B" title= "CLUT:1155EE44">&nbsp;</td>
<td bgcolor="#11BC63" title= "CLUT:1144EE55">&nbsp;</td>
</tr><tr>
<td bgcolor="#1D6930" title= "CLUT:22335544">&nbsp;</td>
<td bgcolor="#509C63" title= "CLUT:33446655">&nbsp;</td>
<td bgcolor="#27902D" title= "CLUT:22336655">&nbsp;</td>
<td bgcolor="#30AC63" title= "CLUT:22447755">&nbsp;</td>
<td bgcolor="#83CF96" title= "CLUT:44557766">&nbsp;</td>
<td bgcolor="#07A02D" title= "CLUT:11337755">&nbsp;</td>
<td bgcolor="#5AC360" title= "CLUT:33447766">&nbsp;</td>
<td bgcolor="#30B72B" title= "CLUT:22337766">&nbsp;</td>
<td bgcolor="#63DF96" title= "CLUT:3355EE66">&nbsp;</td>
<td bgcolor="#3AD360" title= "CLUT:2244EE66">&nbsp;</td>
<td bgcolor="#8DF693" title= "CLUT:4455EE77">&nbsp;</td>
<td bgcolor="#11C72B" title= "CLUT:1133EE66">&nbsp;</td>
<td bgcolor="#63EA5E" title= "CLUT:3344EE77">&nbsp;</td>
<td bgcolor="#3ADE28" title= "CLUT:2233EE77">&nbsp;</td>
</tr><tr>
<td bgcolor="#3C5930" title= "CLUT:33334444">&nbsp;</td>
<td bgcolor="#6F8C63" title= "CLUT:44445555">&nbsp;</td>
<td bgcolor="#46802D" title= "CLUT:33335555">&nbsp;</td>
<td bgcolor="#A2BF96" title= "CLUT:55556666">&nbsp;</td>
<td bgcolor="#79B360" title= "CLUT:44446666">&nbsp;</td>
<td bgcolor="#50A72B" title= "CLUT:33336666">&nbsp;</td>
<td bgcolor="#D5F2C9" title= "CLUT:66667777">&nbsp;</td>
<td bgcolor="#ACE693" title= "CLUT:55557777">&nbsp;</td>
<td bgcolor="#83DA5E" title= "CLUT:44447777">&nbsp;</td>
<td bgcolor="#5ACE28" title= "CLUT:33337777">&nbsp;</td>
<td bgcolor="#63F526" title= "CLUT:3333EEEE">&nbsp;</td>
</tr><tr>
<td bgcolor="#6F972B" title= "CLUT:44335566">&nbsp;</td>
<td bgcolor="#A2CA5E" title= "CLUT:55446677">&nbsp;</td>
<td bgcolor="#79BE28" title= "CLUT:44336677">&nbsp;</td>
<td bgcolor="#A2D526" title= "CLUT:553366EE">&nbsp;</td>
<td bgcolor="#D5FD91" title= "CLUT:665577EE">&nbsp;</td>
<td bgcolor="#ACF15B" title= "CLUT:554477EE">&nbsp;</td>
<td bgcolor="#83E526" title= "CLUT:443377EE">&nbsp;</td>
</tr></table>
</body></html>
}